import React, {useCallback, useContext, useMemo, useRef} from "react";
import {CurtainUuidType, useCurtains} from "./Curtains";
import {classes} from "../../Utils";

/**
 * Current curtain data and functions.
 */
export interface CurtainContextState
{
	/**
	 * Curtain UUID.
	 */
	uuid: CurtainUuidType;

	/**
	 * Close the curtain.
	 */
	close: () => void;

	/**
	 * True if the curtain is closed (while transitioning out).
	 */
	closed: boolean;
}

/**
 * Current curtain context.
 */
const CurtainContext = React.createContext<CurtainContextState>({
	// Empty values.
	uuid: "",
	close: () => {},
	closed: false,
});

/**
 * Hook to access current curtain data and functions.
 */
export function useCurtain(): CurtainContextState
{
	return useContext(CurtainContext);
}

/**
 * Component of an opened curtain instance.
 */
export function CurtainInstance({uuid, children}: React.PropsWithChildren<{
	/**
	 * Curtain UUID.
	 */
	uuid: string;
}>)
{
	// Get close curtain function.
	const {close, isClosed} = useCurtains();

	// Initialize close curtain function.
	const closeCurtain = useRef<() => void>();
	closeCurtain.current = useCallback(() => {
		// Close the current curtain.
		close(uuid);
	}, [uuid, close]);

	// Initialize context state from action functions.
	const contextState = useMemo(() => ({
		uuid: uuid,
		close: () => closeCurtain.current(),
		closed: isClosed(uuid),
	}), [uuid, closeCurtain, isClosed]);

	return (
		<CurtainContext.Provider value={contextState}>
			<div className={classes("curtain", isClosed(uuid) ? "closed" : undefined)}>
				{children}
			</div>
		</CurtainContext.Provider>
	);
}

